import { ArnHostnameParams, BucketHostnameParams } from "./bucketHostnameUtils";
export interface BucketHostname {
  hostname: string;
  bucketEndpoint: boolean;
  signingRegion?: string;
  signingService?: string;
}
export declare const bucketHostname: (
  options: BucketHostnameParams | ArnHostnameParams
) => BucketHostname;
